function [rd,rf]=automatic_signed(grocer_namey,grocer_listx,grocer_signx,varargin)

// PURPOSE: automatic selection of a model by least-squares
// regressions as in Hendry and Krozlig, while imposing the
// sign of selected coefficients
// ------------------------------------------------------------
// INPUT:
// * grocer_namey = a time series, a real (nx1) vector or a
// string equal to the name of a time series or a (nx1) real
// vector between quotes
// * grocer_listx = a matrix of exogenous variables, under as a
// (T x k) real matrix or a (T x 1) string vector of names
// * grocer_signx = a (T x 1) vector of -1, 1 or %nan values,
// each value indicating the expected sign of the coefficients,
// -1 for negative, +1 positive and %nan for indifferent.
// * varargin = any option to automatic (see function automatic
// for details
// ------------------------------------------------------------
// OUPTUT:
// * rd = a results tlist with:
//   - rd('meth') = 'signed automatic'
//   - rd('starting automatic') = original automatic results
//     tlist
//   - rd('ending automatic') = final automatic results tlist,
//     that is with all coefficient fo the good sign
//   - rd('removed var') = a string vector, the variables that
//     have been removed
// * rf = the results tlist of the final model
// ------------------------------------------------------------
// Copyright: Eric Dubois 2011
// http://dubois.ensae.net/grocer.html

grocer_lprt=emptystr()
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   grocer_argi=strsubst(varargin(grocer_i),' ','')
   if  part(grocer_argi,1:3) == 'prt' then
      grocer_lprt=grocer_argi
      varargin(grocer_i)=null()
   end
end

rd=tlist(['results';'meth';'starting automatic';'ending automatic';'removed var'],'signed automatic')
removed_var=[]    
done=%f
while ~done
   done=%t
   r=automatic(grocer_namey,grocer_listx,'prt=nothing',varargin(:))
   rd('starting automatic')=r
   rf=r('final model')
   ncomp=size(r('comp'),2)
   rf_tstat=rf('tstat')(ncomp+1:$)
   rf_namex=rf('namex')(ncomp+1:$)
   for j=1:size(rf_namex,2)
      ind_namex_j=find(grocer_listx == rf_namex(j))
      rf_tstat(j)=rf_tstat(j)*grocer_signx(ind_namex_j)
   end

   if or(rf_tstat < 0) then
      if r('ending reason') == 'stage 2 models selected by bic criterion' then
         // search the stage 2 model with the samallest # of false signs

         r2=r('stage 2 models')
         nmodels=length(r2)/3
         min_tstat=zeros(nmodels,1)
         n_wrongtstat=zeros(nmodels,1)
         name_min_tstat=emptystr(nmodels,1)
         
         for imodel=1:nmodels
            r2_i=r2(3*imodel)
            tstat=r2_i('tstat')(ncomp+1:$)
            name=r2_i('namex')(ncomp+1:$)

            for j=1:size(name,1)
               ind_namex_j=find(grocer_listx == name(j))
               tstat(j)=tstat(j)*grocer_signx(ind_namex_j)
            end
            n_wrongtstat(imodel)=sum(bool2s(tstat<0))
            [min_tstati,ind_min]=min(tstat)
            min_tstat(imodel)=min_tstati
            name_min_tstat(imodel)=name(ind_min)
         end

         good_models=find(min_tstat >= 0)
         ngood_models=size(good_models,2)

         if ~isempty(good_models) then
            crit=zeros(ngood_models,1)
            for imodel=1:ngood_models
               r2_i=r2(3*good_models(imodel))
               execstr('crit(imodel)=r2_i('''+r('criterion')+''')')
            end
            [junk,idef]=max(crit)
            rf=r2(3*good_models(idef))

         else
            ind=find(n_wrongtstat == 1)
            if ~isempty(ind) then
                // select only models with 1 wrong sign
               min_tstat=min_tstat(ind)
            end
            [junk,best_model]=max(min_tstat)
            remove=name_min_tstat(ind(best_model))
            removed_var=[removed_var ; remove]
            ind_listx=find(grocer_listx == remove)
            grocer_listx(ind_listx)=[]
            grocer_signx(ind_listx)=[]
            done=%f

         end

      else
         [junk,min_tstat]=min(rf_tstat)
         remove=find(grocer_listx == rf_namex(min_tstat))
         removed_var=[removed_var ; rf_namex(min_tstat)]
         grocer_listx(remove)=[]
         grocer_signx(remove)=[]
         done=%f
         
      end
   end
         
end

prtauto_signed(r,emptystr(),%io(2))
  
endfunction